using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.Json;
using System.Text.Json.Serialization;
using Microsoft.Extensions.Configuration;

namespace FileExposer.Models;

public record AppSettings(FileWatcherConfiguration FileWatcherConfiguration,
                          SyncConfiguration SyncConfiguration)
{

    public static AppSettings Get(IConfigurationRoot configurationRoot)
    {
        AppSettings result;
#pragma warning disable IL3050, IL2026
        FileWatcherConfiguration? fileWatcherConfiguration = configurationRoot.GetSection(nameof(FileWatcherConfiguration)).Get<FileWatcherConfiguration>();
        SyncConfiguration? syncConfiguration = configurationRoot.GetSection(nameof(SyncConfiguration)).Get<SyncConfiguration>();
#pragma warning restore IL3050, IL2026
        if (fileWatcherConfiguration is null
            || syncConfiguration is null
            || fileWatcherConfiguration?.Company is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
        result = new(fileWatcherConfiguration,
                     syncConfiguration);
        Verify(result);
        return result;
    }

    private static void Verify(AppSettings appSettings)
    {
        if (appSettings.FileWatcherConfiguration.MaxDegreeOfParallelism > Environment.ProcessorCount)
            throw new Exception($"Environment.ProcessorCount must be larger or equal to {nameof(appSettings.FileWatcherConfiguration.MaxDegreeOfParallelism)}");
    }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, AppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class AppSettingsSourceGenerationContext : JsonSerializerContext
{
}